import {Component, Input, OnInit} from '@angular/core';
import {MatTableDataSource} from "@angular/material/table";
import {ScriptAccount} from "../../../../project/game-model/scriptAccounts/ScriptAccount";
import {Transition} from "../../../../project/game-model/gamesystems/transitions/Transition";
import {ScriptAccountAction} from "../../../../project/game-model/gamesystems/actions/ScriptAccountAction";
import {TemplateElement} from "../../../../project/game-model/templates/TemplateElement";
import {
  SimpleTemplateTransition
} from "../../../../project/game-model/templates/simpleGamesystem/SimpleTemplateTransition";

@Component({
  selector: 'app-scriptaccount-action-editor',
  templateUrl: './scriptaccount-action-editor.component.html',
  styleUrl: './scriptaccount-action-editor.component.scss'
})
export class ScriptaccountActionEditorComponent  implements OnInit{
  @Input() transition: Transition<any> | undefined
  @Input() scriptAccounts: ScriptAccount[] = []
  @Input() enableEditing: boolean = false;
  @Input() templateElement: TemplateElement | undefined

  dataSource: MatTableDataSource<ScriptAccountAction> = new MatTableDataSource();
  displayedColumns: string[] = ['scriptAccount', "valueChange", 'edit', 'delete'];

  editedAction: ScriptAccountAction | undefined
  addedAction: ScriptAccountAction | undefined

  ngOnInit() {
    this.dataSource.data = this.getDisplayedActions()



    if(!this.enableEditing) {
      this.displayedColumns = this.displayedColumns.slice(0, -2);
    }
  }

  editAction(scriptAccountAction: ScriptAccountAction) {
    this.editedAction = scriptAccountAction;
  }

  createNewAction() {
    this.addedAction = new ScriptAccountAction(new ScriptAccount("", ""), 0);
    this.editedAction = this.addedAction;

    this.dataSource.data = this.dataSource.data.concat(this.addedAction);
  }

  finishEditing() {
    if(this.addedAction != undefined && this.addedAction.scriptAccount.componentName !== '') {
      if(this.templateElement != undefined && this.transition instanceof SimpleTemplateTransition) {
        if(this.transition.actionMap.has(this.templateElement!)) {
          this.transition.actionMap.get(this.templateElement!)!.push(this.addedAction)
        } else {
          this.transition.actionMap.set(this.templateElement!, [this.addedAction])
        }
      } else {
        this.transition?.addScriptAccountAction(this.addedAction)
      }


      this.dataSource.data = this.getDisplayedActions();
      this.addedAction = undefined;
    }
    this.editedAction = undefined;
  }

  getDisplayedActions(): ScriptAccountAction[] {
    if(this.templateElement == undefined) {
      return this.transition!.scriptAccountActions.map(action => action);
    } else if(this.transition instanceof SimpleTemplateTransition) {
      return this.transition!.actionMap.get(this.templateElement)!
    } else {
      return []
    }
  }

  deleteAction(action: ScriptAccountAction) {
    if(this.templateElement != undefined && this.transition instanceof SimpleTemplateTransition && this.transition.actionMap.has(this.templateElement!)) {
      const updatedAction = this.transition.actionMap.get(this.templateElement)!.filter(currentAction =>
        currentAction.scriptAccount !== action.scriptAccount)
      this.transition.actionMap.set(this.templateElement!, updatedAction)
    } else {
      this.transition!.removeScriptAccountAction(action.scriptAccount)
    }

    this.dataSource.data = this.getDisplayedActions()
  }
}
